const callsites = require('./callsites')
const locationFromCallSite = require('./locationFromCallSite')
const { defaultFilterFn, defaultFilenameFn } = require('./defaultOpt')

module.exports = (err, filterFn = defaultFilterFn, filenameFn = defaultFilenameFn) => {
  const funclocArr = []
  let maxFuncLength = 0

  for (const callsite of callsites(err)) {
    const loc = locationFromCallSite(callsite, filterFn, filenameFn)
    if (!loc) {
      continue
    }

    let func = callsite.getFunctionName() || callsite.getMethodName()
    func = func ? func.replace(/\S+\./g, '') : '<anonnymous>'

    funclocArr.push({ func, loc })

    if (func.length > maxFuncLength) {
      maxFuncLength = func.length
    }
  }

  if (!funclocArr.length) {
    return ''
  }

  const lineArr = []
  for (const funcloc of funclocArr) {
    funcloc.func = funcloc.func.padEnd(maxFuncLength, ' ')
    lineArr.push(`    at ${funcloc.func} ${funcloc.loc}`)
  }

  return lineArr.join('\n')
}
